@@ -8,6 +8,7 @@
 #include "common.h"
 #include "sys.h"
 #include "res.h"
+#include "numeric.h"
 #include "h.h"
 
 #include <signal.h>
@@ -17,7 +18,7 @@
 #include "resolv.h"
 
 #ifndef lint
-static  char sccsid[] = "@(#)res.c	2.30 10 Sep 1993 (C) 1992 Darren Reed";
+static  char sccsid[] = "@(#)res.c	2.34 03 Nov 1993 (C) 1992 Darren Reed";
 #endif
 
 #undef	DEBUG	/* because there is a lot of debug code in here :-) */
@@ -84,6 +85,9 @@ int	op;
 {
 	int	ret = 0;
 
+#ifdef	LRAND48
+	srand48(time(NULL));
+#endif
 	if (op & RES_INITLIST)
 	    {
 		bzero((char *)&reinfo, sizeof(reinfo));
@@ -160,13 +164,13 @@ ResRQ	*old;
 #endif
 	r2ptr = old;
 	if (r2ptr->he.h_name)
-		(void)free((char *)r2ptr->he.h_name);
+		MyFree((char *)r2ptr->he.h_name);
 	for (i = 0; i < MAXALIASES; i++)
 		if ((s = r2ptr->he.h_aliases[i]))
-			(void)free(s);
+			MyFree(s);
 	if (r2ptr->name)
-		(void)free(r2ptr->name);
-	(void)free(r2ptr);
+		MyFree(r2ptr->name);
+	MyFree(r2ptr);
 
 	return;
 }
@@ -183,14 +187,14 @@ Link	*lp;
 	bzero((char *)nreq, sizeof(ResRQ));
 	nreq->next = NULL; /* where NULL is non-zero ;) */
 	nreq->sentat = time(NULL);
-	nreq->retries = 4;
+	nreq->retries = 3;
 	nreq->resend = 1;
 	nreq->srch = -1;
 	if (lp)
 		bcopy((char *)lp, (char *)&nreq->cinfo, sizeof(Link));
 	else
 		bzero((char *)&nreq->cinfo, sizeof(Link));
-	nreq->timeout = nreq->sentat + RES_TIMEOUT;
+	nreq->timeout = 4;	/* start at 4 and exponential inc. */
 	nreq->he.h_addrtype = AF_INET;
 	nreq->he.h_name = NULL;
 	nreq->he.h_aliases[0] = NULL;
@@ -206,14 +210,15 @@ time_t	timeout_query_list(now)
 time_t	now;
 {
 	Reg1	ResRQ	*rptr, *r2ptr;
-	Reg2	time_t	next = 0;
+	Reg2	time_t	next = 0, tout;
 	aClient	*cptr;
 
 	Debug((DEBUG_DNS,"timeout_query_list at %s",myctime(now)));
 	for (rptr = first; rptr; rptr = r2ptr)
 	    {
 		r2ptr = rptr->next;
-		if (now >= rptr->timeout)
+		tout = rptr->sentat + rptr->timeout;
+		if (now >= tout)
 			if (--rptr->retries <= 0)
 			    {
 #ifdef DEBUG
@@ -249,7 +254,7 @@ time_t	now;
 			else
 			    {
 				rptr->sentat = now;
-				rptr->timeout = now + RES_TIMEOUT;
+				rptr->timeout += rptr->timeout;
 				resend_query(rptr);
 #ifdef DEBUG
 				Debug((DEBUG_INFO,"r %x now %d retry %d c %x",
@@ -257,8 +262,8 @@ time_t	now;
 					rptr->cinfo.value.cptr));
 #endif
 			    }
-		if (!next || rptr->timeout < next)
-			next = rptr->timeout;
+		if (!next || tout < next)
+			next = tout;
 	    }
 	return (next > now) ? next : (now + AR_TTL);
 }
@@ -378,23 +383,27 @@ Reg1	ResRQ	*rptr;
 	(void)strncpy(hname, name, sizeof(hname)-1);
 	len = strlen(hname);
 
-	if (rptr && (hname[len-1] != '.'))
+	if (rptr && !index(hname, '.') && _res.options & RES_DEFNAMES)
 	    {
 		(void)strncat(hname, dot, sizeof(hname)-len-1);
+		len++;
+		(void)strncat(hname, _res.defdname, sizeof(hname) - len -1);
+	    }
+	else if (rptr && (hname[len-1] != '.'))
+	    {
+		(void)strncat(hname, dot, sizeof(hname)-len-1);
+		len++;
 		/*
 		 * NOTE: The logical relationship between DNSRCH and DEFNAMES
 		 * is implies. ie no DEFNAMES, no DNSRCH.
+		 * Or So I thought...perhaps not anymore...
 		 */
-		if (_res.options & (RES_DEFNAMES|RES_DNSRCH) ==
-		    (RES_DEFNAMES|RES_DNSRCH))
+		if (_res.options & RES_DNSRCH)
 		    {
 			if (_res.dnsrch[rptr->srch])
 				(void)strncat(hname, _res.dnsrch[rptr->srch],
-					sizeof(hname) - ++len -1);
-		    }
-		else if (_res.options & RES_DEFNAMES)
-			(void)strncat(hname, _res.defdname,
 					sizeof(hname) - len -1);
+		    }
 	    }
 
 	/*
@@ -461,10 +470,16 @@ ResRQ	*rptr;
 		return r;
 	    }
 	hptr = (HEADER *)buf;
+#ifdef GETTIMEOFDAY
 	(void) gettimeofday(&tv, NULL);
 	do {
 		hptr->id = htons(ntohs(hptr->id) + k +
 				 (u_short)(tv.tv_usec & 0xffff));
+#endif /* GETTIMEOFDAY */
+#ifdef LRAND48
+        do {
+		hptr->id = htons(ntohs(hptr->id) + k + lrand48() & 0xffff);
+#endif /* LRAND48 */
 		k++;
 	} while (find_id(ntohs(hptr->id)));
 	rptr->id = ntohs(hptr->id);
@@ -1205,7 +1220,7 @@ aCache	*ocp;
 	** Cleanup any references to this structure by destroying the
 	** pointer.
 	*/
-	for (hashv = 0; hashv < highest_fd; hashv++)
+	for (hashv = highest_fd; hashv >= 0; hashv--)
 		if ((cptr = local[hashv]) && (cptr->hostp == hp))
 			cptr->hostp = NULL;
 	/*
@@ -1253,12 +1268,12 @@ aCache	*ocp;
 	 * of alias pointers.
 	 */
 	if (hp->h_name)
-		(void)free(hp->h_name);
+		MyFree(hp->h_name);
 	if (hp->h_aliases)
 	    {
 		for (hashv = 0; hp->h_aliases[hashv]; hashv++)
-			(void)free(hp->h_aliases[hashv]);
-		(void)free((char *)hp->h_aliases);
+			MyFree(hp->h_aliases[hashv]);
+		MyFree((char *)hp->h_aliases);
 	    }
 
 	/*
@@ -1267,11 +1282,11 @@ aCache	*ocp;
 	if (hp->h_addr_list)
 	    {
 		if (*hp->h_addr_list)
-			(void)free((char *)*hp->h_addr_list);
-		(void)free((char *)hp->h_addr_list);
+			MyFree((char *)*hp->h_addr_list);
+		MyFree((char *)hp->h_addr_list);
 	    }
 
-	(void)free((char *)ocp);
+	MyFree((char *)ocp);
 
 	incache--;
 	cainfo.ca_dels++;
@@ -1354,3 +1369,39 @@ char	*parv[];
 		   reinfo.re_resends, reinfo.re_timeouts);
 	return 0;
 }
+
+u_long	cres_mem(sptr)
+aClient	*sptr;
+{
+	register aCache	*c = cachetop;
+	register struct	hostent	*h;
+	register int	i;
+	u_long	nm = 0, im = 0, sm = 0, ts = 0;
+
+	for ( ;c ; c = c->list_next)
+	    {
+		sm += sizeof(*c);
+		h = &c->he;
+		for (i = 0; h->h_addr_list[i]; i++)
+		    {
+			im += sizeof(char *);
+			im += sizeof(struct in_addr);
+		    }
+		im += sizeof(char *);
+		for (i = 0; h->h_aliases[i]; i++)
+		    {
+			nm += sizeof(char *);
+			nm += strlen(h->h_aliases[i]);
+		    }
+		nm += i - 1;
+		nm += sizeof(char *);
+		if (h->h_name)
+			nm += strlen(h->h_name);
+	    }
+	ts = ARES_CACSIZE * sizeof(CacheTable);
+	sendto_one(sptr, ":%s %d %s :RES table %d",
+		   me.name, RPL_STATSDEBUG, sptr->name, ts);
+	sendto_one(sptr, ":%s %d %s :Structs %d IP storage %d Name storage %d",
+		   me.name, RPL_STATSDEBUG, sptr->name, sm, im, nm);
+	return ts + sm + im + nm;
+}
