@@ -22,7 +22,7 @@
  */
 
 #ifndef lint
-static  char sccsid[] = "@(#)s_user.c	2.62 15 Sep 1993 (C) 1988 University of Oulu, \
+static  char sccsid[] = "@(#)s_user.c	2.68 07 Nov 1993 (C) 1988 University of Oulu, \
 Computing Center and Jarkko Oikarinen";
 #endif
 
@@ -245,6 +245,52 @@ char	*nick;
 }
 
 
+/*
+** canonize
+**
+** reduce a string of duplicate list entries to contain only the unique
+** items.  Unavoidably O(n^2).
+*/
+char	*canonize(buffer)
+char	*buffer;
+{
+	static	char	cbuf[BUFSIZ];
+	register char	*s, *t, *cp = cbuf;
+	register int	l = 0;
+	char	*p = NULL, *p2;
+
+	*cp = '\0';
+
+	for (s = strtoken(&p, buffer, ","); s; s = strtoken(&p, NULL, ","))
+	    {
+		if (l)
+		    {
+			for (p2 = NULL, t = strtoken(&p2, cbuf, ","); t;
+			     t = strtoken(&p2, NULL, ","))
+				if (!mycmp(s, t))
+					break;
+				else if (p2)
+					p2[-1] = ',';
+		    }
+		else
+			t = NULL;
+		if (!t)
+		    {
+			if (l)
+				*(cp-1) = ',';
+			else
+				l = 1;
+			(void)strcpy(cp, s);
+			if (p)
+				cp += (p - s);
+		    }
+		else if (p2)
+			p2[-1] = ',';
+	    }
+	return cbuf;
+}
+
+
 /*
 ** register_user
 **	This function is called when both NICK and USER messages
@@ -288,7 +334,7 @@ char	*nick, *username;
 			sendto_ops("Received unauthorized connection from %s.",
 				   get_client_name(sptr,FALSE));
 			ircstp->is_ref++;
-			return exit_client(cptr, sptr, sptr,
+			return exit_client(cptr, sptr, &me,
 					   "No Authorization");
 		      } 
 		if (IsUnixSocket(sptr))
@@ -320,7 +366,7 @@ char	*nick, *username;
 			ircstp->is_ref++;
 			sendto_one(sptr, err_str(ERR_PASSWDMISMATCH),
 				   me.name, parv[0]);
-			return exit_client(cptr, sptr, sptr, "Bad Password");
+			return exit_client(cptr, sptr, &me, "Bad Password");
 		    }
 		bzero(sptr->passwd, sizeof(sptr->passwd));
 		/*
@@ -339,7 +385,7 @@ char	*nick, *username;
 		    }
 #endif
 		if (oldstatus == STAT_MASTER && MyConnect(sptr))
-			m_oper(&me, sptr, 1, parv);
+			(void)m_oper(&me, sptr, 1, parv);
 	    }
 	else
 		strncpyzt(sptr->user->username, username, USERLEN+1);
@@ -532,7 +578,8 @@ char	*parv[];
 		** send error reply and ignore the command.
 		*/
 		sendto_one(sptr, err_str(ERR_NICKNAMEINUSE),
-			   me.name, parv[0], nick);
+			   /* parv[0] is empty when connecting */
+			   me.name, BadPtr(parv[0]) ? "*" : parv[0], nick);
 		return 0; /* NICK message ignored */
 	    }
 	/*
@@ -702,6 +749,8 @@ int	notice;
 		return -1;
 	    }
 
+	if (MyConnect(sptr))
+		parv[1] = canonize(parv[1]);
 	for (p = NULL, nick = strtoken(&p, parv[1], ","); nick;
 	     nick = strtoken(&p, NULL, ","))
 	    {
@@ -1024,6 +1073,7 @@ char	*parv[];
 		NULL,	/* away */
 		0,	/* last */
 		1,      /* refcount */
+		0,	/* joined */
 		"<Unknown>",	/* user */
 		"<Unknown>",	/* host */
 		"<Unknown>"	/* server */
@@ -1034,9 +1084,9 @@ char	*parv[];
 	aChannel *chptr;
 	char	*nick, *tmp, *name;
 	char	*p = NULL;
-	int	found;
+	int	found, len, mlen;
 
-	if (check_registered(sptr))
+	if (check_registered_user(sptr))
 		return 0;
 
     	if (parc < 2)
@@ -1116,29 +1166,35 @@ char	*parv[];
 				   parv[0], name,
 				   user->username, user->host, acptr->info);
 			found = 1;
-			for (buf[0] = '\0', lp = user->channel; lp;
+			mlen = strlen(me.name) + strlen(parv[0]) + 6 +
+				strlen(name);
+			for (len = 0, *buf = '\0', lp = user->channel; lp;
 			     lp = lp->next)
 			    {
 				chptr = lp->value.chptr;
 				if (ShowChannel(sptr, chptr))
 				    {
-					if (strlen(buf) +
-					    strlen(chptr->chname)
-                                            > (size_t) 400)
+					if (len + strlen(chptr->chname)
+                                            > (size_t) BUFSIZE - 4 - mlen)
 					    {
 						sendto_one(sptr,
 							   ":%s %d %s %s :%s",
 							   me.name,
 							   RPL_WHOISCHANNELS,
 							   parv[0], name, buf);
-						buf[0] = '\0';
+						*buf = '\0';
+						len = 0;
 					    }
 					if (is_chan_op(acptr, chptr))
-						(void)strcat(buf, "@");
+						*(buf + len++) = '@';
 					else if (has_voice(acptr, chptr))
-						(void)strcat(buf, "+");
-					(void)strcat(buf, chptr->chname);
-					(void)strcat(buf, " ");
+						*(buf + len++) = '+';
+					if (len)
+						*(buf + len) = '\0';
+					(void)strcpy(buf + len, chptr->chname);
+					len += strlen(chptr->chname);
+					(void)strcat(buf + len, " ");
+					len++;
 				    }
 			    }
 			if (buf[0] != '\0')
@@ -1368,9 +1424,7 @@ char	*parv[];
 		if (!BadPtr(path))
 		    {
 			(void)sprintf(buf, "%s%s (%s)",
-				cptr->name,
-				IsOper(sptr) ? "" : "(L)",
-				path);
+				cptr->name, IsOper(sptr) ? "" : "(L)", path);
 			path = buf;
 		    }
 		else
@@ -1487,7 +1541,7 @@ char	*parv[];
 
 		if (away)
 		    {
-			(void)free(away);
+			MyFree(away);
 			sptr->user->away = NULL;
 		    }
 		sendto_serv_butone(cptr, ":%s AWAY", parv[0]);
@@ -1632,11 +1686,13 @@ char	*parv[];
 	extern	char *crypt();
 #endif /* CRYPT_OPER_PASSWORD */
 
+	if (check_registered_user(sptr))
+		return 0;
 
 	name = parc > 1 ? parv[1] : NULL;
 	password = parc > 2 ? parv[2] : NULL;
 
-	if (MyClient(sptr) && (BadPtr(name) || BadPtr(password)))
+	if (!IsServer(cptr) && (BadPtr(name) || BadPtr(password)))
 	    {
 		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS),
 			   me.name, parv[0], "OPER");
@@ -1677,12 +1733,12 @@ char	*parv[];
         /* use first two chars of the password they send in as salt */
 
         /* passwd may be NULL. Head it off at the pass... */
-        salt[0]='\0';
-        if(password)
+        salt[0] = '\0';
+        if (password && aconf->passwd)
 	    {
-        	salt[0]=aconf->passwd[0];
-		salt[1]=aconf->passwd[1];
-		salt[2]='\0';
+        	salt[0] = aconf->passwd[0];
+		salt[1] = aconf->passwd[1];
+		salt[2] = '\0';
 		encr = crypt(password, salt);
 	    }
 	else
@@ -1709,7 +1765,8 @@ char	*parv[];
 		else
 			SetOper(sptr);
 		*--s =  '@';
-		sendto_ops("%s is now operator (%c)", parv[0],
+		sendto_ops("%s (%s@%s) is now operator (%c)", parv[0],
+			   sptr->user->username, sptr->user->host,
 			   IsOper(sptr) ? 'O' : 'o');
 		sptr->flags |= (FLAGS_SERVNOTICE|FLAGS_WALLOP);
 		send_umode_out(cptr, sptr, old);
@@ -1725,7 +1782,6 @@ char	*parv[];
 #endif
 #ifdef FNAME_OPERLOG
 	      {
-                char    linebuf[160];
                 int     logfile;
 
                 /*
@@ -1741,13 +1797,12 @@ char	*parv[];
                     (logfile = open(FNAME_OPERLOG, O_WRONLY|O_APPEND)) != -1)
 		{
 		  (void)alarm(0);
-                        (void)sprintf(linebuf,
-				      "%s OPER (%s) (%s) by (%s!%s@%s)\n",
+                        (void)sprintf(buf, "%s OPER (%s) (%s) by (%s!%s@%s)\n",
 				      myctime(time(NULL)), name, encr,
 				      parv[0], sptr->user->username,
 				      sptr->sockhost);
 		  (void)alarm(3);
-		  (void)write(logfile, linebuf, strlen(linebuf));
+		  (void)write(logfile, buf, strlen(buf));
 		  (void)alarm(0);
 		  (void)close(logfile);
 		}
@@ -1810,17 +1865,17 @@ aClient *cptr, *sptr;
 int	parc;
 char	*parv[];
 {
-	char	uhbuf[USERHOST_REPLYLEN], *p = NULL;
+	char	*p = NULL;
 	aClient	*acptr;
 	Reg1	char	*s;
 	Reg2	int	i, len;
 
-	if (parc > 2)
-		(void)m_userhost(cptr, sptr, parc-1, parv+1);
-
 	if (check_registered(sptr))
 		return 0;
 
+	if (parc > 2)
+		(void)m_userhost(cptr, sptr, parc-1, parv+1);
+
 	if (parc < 2)
 	    {
 		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS),
@@ -1830,22 +1885,22 @@ char	*parv[];
 
 	(void)sprintf(buf, rpl_str(RPL_USERHOST), me.name, parv[0]);
 	len = strlen(buf);
-	*uhbuf = '\0';
+	*buf2 = '\0';
 
 	for (i = 5, s = strtoken(&p, parv[1], " "); i && s;
 	     s = strtoken(&p, (char *)NULL, " "), i--)
 		if ((acptr = find_person(s, NULL)))
 		    {
-			if (*uhbuf)
+			if (*buf2)
 				(void)strcat(buf, " ");
-			(void)sprintf(uhbuf, "%s%s=%c%s@%s",
+			(void)sprintf(buf2, "%s%s=%c%s@%s",
 				acptr->name,
 				IsAnOper(acptr) ? "*" : "",
 				(acptr->user->away) ? '-' : '+',
 				acptr->user->username,
 				acptr->user->host);
-			(void)strncat(buf, uhbuf, sizeof(buf) - len);
-			len += strlen(uhbuf);
+			(void)strncat(buf, buf2, sizeof(buf) - len);
+			len += strlen(buf2);
 		    }
 	sendto_one(sptr, "%s", buf);
 	return 0;
@@ -1922,7 +1977,7 @@ char	*parv[];
 	nbuf[i] = 0;
 
 	if (IsOper(sptr) && wilds)
-		for (i = 0; i <= highest_fd; i++)
+		for (i = highest_fd; i >= 0; i--)
 		    {
 			if (!(acptr = local[i]))
 				continue;
@@ -1965,13 +2020,12 @@ char	*parv[];
 	Reg2	int	*s;
 	Reg3	char	**p, *m;
 	aClient	*acptr;
-	char	flagbuf[20];
 	int	what, setflags;
 
 	if (check_registered_user(sptr))
 		return 0;
 
-	what = MODE_NULL;
+	what = MODE_ADD;
 
 	if (parc < 2)
 	    {
@@ -2003,14 +2057,15 @@ char	*parv[];
  
 	if (parc < 3)
 	    {
-		m = flagbuf;
+		m = buf;
 		*m++ = '+';
-		for (s = user_modes; (flag = *s) && (m < flagbuf+18); s += 2)
+		for (s = user_modes; (flag = *s) && (m - buf < BUFSIZE - 4);
+		     s += 2)
 			if (sptr->flags & flag)
 				*m++ = (char)(*(s+1));
 		*m = '\0';
 		sendto_one(sptr, rpl_str(RPL_UMODEIS),
-			   me.name, parv[0], flagbuf);
+			   me.name, parv[0], buf);
 		return 0;
 	    }
 
@@ -2143,21 +2198,19 @@ void	send_umode_out(cptr, sptr, old)
 aClient *cptr, *sptr;
 int	old;
 {
-	char	flagbuf[20];
 	Reg1    int     i;
 	Reg2    aClient *acptr;
 
-	send_umode(NULL, sptr, old, SEND_UMODES, flagbuf);
+	send_umode(NULL, sptr, old, SEND_UMODES, buf);
 # ifdef NPATH
-        check_command((long)4, ":%s MODE %s :%s", sptr->name, 
-                      sptr->name, flagbuf);
+        check_command((long)4, ":%s MODE %s :%s", sptr->name, sptr->name, buf);
 # endif
-	for (i = 0; i <= highest_fd; i++)
+	for (i = highest_fd; i >= 0; i--)
 		if ((acptr = local[i]) && IsServer(acptr) &&
-		    (acptr != cptr) && (acptr != sptr) && *flagbuf)
+		    (acptr != cptr) && (acptr != sptr) && *buf)
 			sendto_one(acptr, ":%s MODE %s :%s",
-				   sptr->name, sptr->name, flagbuf);
+				   sptr->name, sptr->name, buf);
 
 	if (cptr && MyClient(cptr))
-		send_umode(cptr, sptr, old, ALL_UMODES, flagbuf);
+		send_umode(cptr, sptr, old, ALL_UMODES, buf);
 }
